@@ -5,9 +5,14 @@ from django.conf import settings
 from django.utils import importlib
 from django.contrib.contenttypes import generic
 from django.contrib.contenttypes.models import ContentType
-from django.db.models.loading import cache, get_model
+import django
+from django.db.models.loading import get_model
+if django.VERSION >= (1, 7):
+    from django.apps import apps
+else:
+    from django.db.models.loading import cache
 from django.db.models.base import ModelBase
-from django.db.models import (\
+from django.db.models import (
     CharField, EmailField, SlugField, TextField, URLField,
     DateField, DateTimeField, TimeField,
     AutoField, IntegerField, SmallIntegerField,
@@ -15,36 +20,16 @@ from django.db.models import (\
     BooleanField, DecimalField, FloatField,
     FileField, ImageField, Field,
     ForeignKey, ManyToManyField, OneToOneField)
-
+from django.db.models.fields.related import ForeignRelatedObjectsDescriptor
 try:
     from django.db.models import BigIntegerField
 except ImportError:
     BigIntegerField = IntegerField
 
-import generators
-from exceptions import ModelNotFound, AmbiguousModelName, InvalidQuantityException
-
-from six import string_types
-
-
-class Sequence(object):
-
-    def __init__(self, value, increment_by=1):
-        self.value = value
-        self.counter = increment_by
-        self.increment_by = increment_by
-
-    def get_inc(self, model):
-        if not model.objects.count():
-            self.counter = self.increment_by
-        i = self.counter
-        self.counter += self.increment_by
-        return i
-
-    def gen(self, model):
-        inc = self.get_inc(model)
-        return self.value + type(self.value)(inc)
+from . import generators
+from .exceptions import ModelNotFound, AmbiguousModelName, InvalidQuantityException, RecipeIteratorEmpty
 
+from six import string_types, advance_iterator, PY3
 
 recipes = None
 
@@ -158,11 +143,15 @@ class ModelFinder(object):
         :param name String on the form 'applabel.modelname' or 'modelname'.
         :return a model class.
         '''
-        if '.' in name:
-            app_label, model_name = name.split('.')
-            model =  get_model(app_label, model_name)
-        else:
-            model = self.get_model_by_name(name)
+        try:
+            if '.' in name:
+                app_label, model_name = name.split('.')
+                model = get_model(app_label, model_name)
+            else:
+                model = self.get_model_by_name(name)
+        except LookupError:  # Django 1.7.0a1 throws an exception
+            # Lower djangos just fail silently
+            model = None
 
         if not model:
             raise ModelNotFound("Could not find model '%s'." % name.title())
@@ -194,7 +183,12 @@ class ModelFinder(object):
         unique_models = {}
         ambiguous_models = []
 
-        for app_model in cache.app_models.values():
+        if django.VERSION >= (1, 7):
+            all_models = apps.all_models
+        else:
+            all_models = cache.app_models
+
+        for app_model in all_models.values():
             for name, model in app_model.items():
                 if name not in unique_models:
                     unique_models[name] = model
@@ -202,12 +196,19 @@ class ModelFinder(object):
                     ambiguous_models.append(name)
 
         for name in ambiguous_models:
-            unique_models.pop(name)
+            unique_models.pop(name, None)
 
         self._ambiguous_models = ambiguous_models
         self._unique_models = unique_models
 
 
+def is_iterator(value):
+    if PY3:
+        return hasattr(value, '__next__')
+    else:
+        return hasattr(value, 'next')
+
+
 class Mommy(object):
     attr_mapping = {}
     type_mapping = None
@@ -244,6 +245,7 @@ class Mommy(object):
         '''Creates, but do not persists, an instance of the model
         associated with Mommy instance.'''
         self.type_mapping[ForeignKey] = prepare
+        self.type_mapping[OneToOneField] = prepare
         return self._make(commit=False, **attrs)
 
     def get_fields(self):
@@ -251,18 +253,25 @@ class Mommy(object):
 
     def _make(self, commit=True, **attrs):
         is_rel_field = lambda x: '__' in x
+        iterator_attrs = dict((k, v) for k, v in attrs.items() if is_iterator(v))
         model_attrs = dict((k, v) for k, v in attrs.items() if not is_rel_field(k))
         self.rel_attrs = dict((k, v) for k, v in attrs.items() if is_rel_field(k))
         self.rel_fields = [x.split('__')[0] for x in self.rel_attrs.keys() if is_rel_field(x)]
 
         for field in self.get_fields():
+
+            # Skip links to parent so parent is not created twice.
+            if isinstance(field, OneToOneField) and field.rel.parent_link:
+                continue
+
             field_value_not_defined = field.name not in model_attrs
 
             if isinstance(field, (AutoField, generic.GenericRelation)):
                 continue
 
             if all([field.name not in model_attrs, field.name not in self.rel_fields, field.name not in self.attr_mapping]):
-                if not issubclass(field.__class__, Field) or field.has_default() or field.blank:
+                # Django is quirky in that BooleanFields are always "blank", but have no default default.
+                if not issubclass(field.__class__, Field) or field.has_default() or (field.blank and not isinstance(field, BooleanField)):
                     continue
 
             if isinstance(field, ManyToManyField):
@@ -275,25 +284,41 @@ class Mommy(object):
                     continue
                 else:
                     model_attrs[field.name] = self.generate_value(field)
-            elif isinstance(model_attrs[field.name], Sequence):
-                model_attrs[field.name] = model_attrs[field.name].gen(self.model)
+            elif callable(model_attrs[field.name]):
+                model_attrs[field.name] = model_attrs[field.name]()
+            elif field.name in iterator_attrs:
+                try:
+                    model_attrs[field.name] = advance_iterator(iterator_attrs[field.name])
+                except StopIteration:
+                    raise RecipeIteratorEmpty('{} iterator is empty.'.format(field.name))
 
         return self.instance(model_attrs, _commit=commit)
 
     def m2m_value(self, field):
-        if not (self.make_m2m or field.null) and field.name not in self.rel_fields:
-            return []
-        else:
+        if field.name in self.rel_fields:
             return self.generate_value(field)
+        if not self.make_m2m or field.null:
+            return []
+        return self.generate_value(field)
 
     def instance(self, attrs, _commit):
+        one_to_many_keys = {}
+        for k in tuple(attrs.keys()):
+            field = getattr(self.model, k, None)
+            if isinstance(field, ForeignRelatedObjectsDescriptor):
+                one_to_many_keys[k] = attrs.pop(k)
         instance = self.model(**attrs)
         # m2m only works for persisted instances
         if _commit:
             instance.save()
+            self._handle_one_to_many(instance, one_to_many_keys)
             self._handle_m2m(instance)
         return instance
 
+    def _handle_one_to_many(self, instance, attrs):
+        for k, v in attrs.items():
+            setattr(instance, k, v)
+
     def _handle_m2m(self, instance):
         for key, values in self.m2m_dict.items():
             if not values:
